package org.eclipse.nebula.animation.effects;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.nebula.animation.movement.IMovement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * 
 *
 */
public class FormLayoutEffect extends AbstractEffect {
	
	Set<Composite> controlParents;	
	Map<Integer,AttachmentChange> attachmentEffects;	

	public void setAttachmentEffect(Integer position, AttachmentChange attachmentEffect) {
		if (attachmentEffects == null) {
			attachmentEffects = new HashMap<Integer,AttachmentChange>();
			controlParents =new HashSet<Composite>();			
		}		
		attachmentEffects.put(position, attachmentEffect);
		controlParents.add(attachmentEffect.control.getParent());
	}	
	
	public FormLayoutEffect(long lengthMilli, IMovement movement, Runnable onStop,	Runnable onCancel) {
		super(lengthMilli, movement, onStop, onCancel);
		easingFunction.init(0, 1, (int) lengthMilli);
	}
	
	protected FormAttachment getFormAttachment(FormData formData, int position) {
    	switch (position) {
    	case SWT.TOP:   
    		return formData.top;
    	case SWT.BOTTOM:                		
    		return formData.bottom;
    	case SWT.LEFT:   
    		return formData.left;
    	case SWT.RIGHT:                		
    		return formData.right;
    	}  	
    	throw new RuntimeException("No form data found for position: "+position);
	}

    public void applyEffect(final long currentTime) {           
        if (attachmentEffects != null) {            	            	            	               
            for (Integer position : attachmentEffects.keySet()) {
            	AttachmentChange data = attachmentEffects.get(position);                	
            	Control control = data.control;
            	if (control.isDisposed()) {
            		break;
            	}
            	Object obj = control.getLayoutData();                	
            	FormData formData = (FormData)obj;                	            	
            	FormAttachment formAttachment = getFormAttachment(formData,position);             		                	                	
            	if (data.offsetDiff != 0) {
            		formAttachment.offset = data.getOffset(easingFunction,(int)currentTime);
        		}
            	if (data.percentDiff != 0) {
            		formAttachment.numerator = data.getPercent(easingFunction,(int)currentTime);
        		}                  	
            }
            for (Composite c : controlParents) {
            	c.layout();
            }

        }
    }
}